/* 
 * Generate fractal landscapes.
 * MAP contains elevation.
 *
 * Basic algorithm: draw a line through the map.
 *                  everything on one side moves up 1
 *                  everything on the other moves down 1.
 * 
 * Fixed for Unix by Peter Keel <killer@discordia.ch>, 06.09.1999
 *
 */
 
#include <stdio.h>
#include <ncurses.h>

int map[61][79];

/*------------------------------------------* 
 *                                          *
 * Routine: fraqtl.                         *
 *                                          *
 * Purpose: Draw fractal landscape.         *
 *                                          *
 * Description: Pick two points (x1,y1),    *
 *              (x2,y2).  Figure the eqn    *
 *              for that line.  Now pick s, *
 *              a random side of the line.  *
 *              Everything on one side of   *
 *              the line moves up one;      *
 *              Everything on the other side*
 *              moves down 1.               *
 *                                          *
 * Caveats: Disallows verticle lines, due to*
 *          problems calculating slope.     *
 *                                          *
 *------------------------------------------*/
void fraqtl(a,b)
int a,b;			/* x size, y size */
{
	int     s,		/* s=1, side above line incremented;
	  			   s=0, side below line incremented. */
	        i,		/* loop counters */
	        j,
	        x1,		/* Point one */
	        y1,
	        x2,		/* Point two */
	        y2,
	        m,		/* slope of line connecting p1 and p2 */
	        p,		/* y-intercept of line */
	        l,		/* loop counter */
	        n,		/* Number of iterations */
	        t;		/* temporary variable */
		

/* 
 * Get number of iterations.  The more you have the better the maps
 */

	printf ("Iterations: ");
	scanf ("%d", &n);

/* 
 * reset map to start
 */

	for (i=0;i<=a;i++) {
		for (j=0;j<=b;j++) {
			map[i][j]=0;
		}
	}

/* 
 * Now do the real work
 */

	for (l = 0; l <= n; l++) {

/* 
 * Pick points a and b, disallowing vertical lines
 */

		x1=rand()%a;
		y1=rand()%b;
		y2=rand()%b;
		x2=rand()%a;
		while (x1 == x2) {
			x2=rand()%a;
		}

/* 
 * Calculate equation of line
 */
 

		m = (y2 - y1) / (x2 - x1);
		p = y1 - (m * x1);

/* 
 * Pick a random side of the line
 */
		s = rand ()%1;

/* 
 * Now, loop through each coordinate on the map.
 */
 
		for (i = 0; i <= a; i++) {
			for (j = 0; j <= b; j++) {

/* 
 * t= the value of the line equation at that point
 */
 
				t = (m * j) + p;
/* 
 * If s=1, increment stuff above the line and decrement stuff
 * below.  Otherwise, do just the opposite.
 */
				if (s) {

					if (i > t) {
						map[i][j]++;
					} else {
						map[i][j]--;
					}
				} else {
					if (i > t) {
						map[i][j]--;
					} else {
						map[i][j]++;
					}

				}
			}
		}

	}
}



/*------------------------------------------* 
 *                                          *
 * Routine: prfrct                          *
 *                                          *
 * Purpose: print fractal map               *
 *                                          *
 * Description: Loop through each point on  *
 *              the map, give it a symbol   *
 *              based on arbitrary height   *
 *              ranges.                     *
 *                                          *
 * Caveats: height ranges are arbitrary and *
 *          expressed as a fraction of max. *
 *          Most always gens a coast, almost*
 *          never the Himalayas             *
 *                                          *
 *------------------------------------------*/
void prfrct(a,b)
int a,b;
{
	int i,j,m;
	
	clear();	/* clear screen */
	
/* 
 * Find maximum height on the map
 */

	m=0;
	for (i=0;i<=a;i++) {
		for (j=0;j<=b;j++) {
			if (map[i][j]>m) m=map[i][j];
		}
	}

/* 
 * Now print border and map.
 * '~' --- Oceans
 * ' ' --- Coastal plains
 * '*' --- Forrest lands
 * '.' --- Highlands, hills
 * '^' --- Mountains
 */
 
	for (i=0;i<=b+1;i++) putchar ('-');
	printf("\n");	
	
	for (i=0;i<=a;i++) {
		putchar('|');
		for (j=0;j<=b;j++) {

			if (map[i][j]<0) putchar('~');

			if ((map[i][j]>=0)&&(map[i][j]<(m/2))) putchar(' ');

			if ((map[i][j]>=(m/2))&&(map[i][j]<(m*2/3))) putchar('*');
			if ((map[i][j]>=(m*2/3))&&(map[i][j]<(m*5/6))) putchar('.');
			if ((map[i][j]>=(m*5/6))) putchar('^');
		}
	printf("|\n");
	}
	for (i=0;i<=b+1;i++) putchar ('-');
	printf("\n");

}

/*------------------------------------------* 
 *                                          *
 * Routine: main                            *
 *                                          *
 * Purpose: Just build fractals forever     *
 *                                          *
 * Description:                             *
 *                                          *
 * Caveats:                                 *
 *                                          *
 *------------------------------------------*/
main ()
{
	srand ();
	while (1) {

		fraqtl (60,76);
		prfrct (60,76);
	}

}
		
	
